"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const ReactModal = require("react-modal");
const react_codemirror2_1 = require("react-codemirror2");
require("codemirror/mode/python/python");
const connection_1 = require("./connection");
const upload_1 = require("./upload");
const kernel_1 = require("./kernel");
require("../style/configure-modal.css");
const apputils_1 = require("@jupyterlab/apputils");
var STEP;
(function (STEP) {
    STEP[STEP["CONFIGURATION"] = 0] = "CONFIGURATION";
    STEP[STEP["INTEGRATION"] = 1] = "INTEGRATION";
})(STEP || (STEP = {}));
class ConfigureModal extends React.Component {
    constructor(props) {
        super(props);
        this.initializeDialog = () => {
            this.localConnection
                .validate()
                .then(() => {
                this.setState({ isApiTokenValid: true });
                Promise.all([
                    this.localConnection
                        .listProjects()
                        .then(projects => {
                        this.setState({
                            projectsList: projects.map(project => `${project.organizationName}/${project.name}`)
                        });
                        return projects;
                    }),
                    this.localConnection
                        .getNotebook()
                        .then(notebook => {
                        this.setState({ notebook });
                        return notebook;
                    })
                ])
                    .then(([projects, notebook]) => {
                    const project = projects.find(project => project.id === notebook.projectId);
                    this.setState({ selectedProject: `${project.organizationName}/${project.name}` });
                });
            })
                .catch(() => this.setState({ isApiTokenValid: false }));
        };
        this.renderApiTokenStatus = () => {
            return this.renderInputStatus(this.state.isApiTokenValid);
        };
        this.renderProjectsSelect = () => {
            const { selectedProject, projectsList } = this.state;
            return (React.createElement("select", { id: "neptune-project", className: "n-form-input n-form-input-with-status", value: selectedProject, disabled: projectsList === null, onChange: this.updateSelectedProject }, (projectsList === null) ? (React.createElement("option", { label: selectedProject, value: selectedProject })) : ([
                React.createElement("option", { value: "", key: "--" }, "--Please choose the project--"),
                ...projectsList.map(project => React.createElement("option", { key: project, label: project, value: project }))
            ])));
        };
        this.renderProjectsStatus = () => {
            return this.renderInputStatus(!!this.state.selectedProject);
        };
        this.getNotebookURI = () => {
            const { selectedProject, notebook } = this.state;
            if (!notebook) {
                return null;
            }
            const { id, name, lastCheckpointId } = notebook;
            const apiAddress = this.localConnection.getApiAddress();
            return `${apiAddress}\/${selectedProject}\/n\/${name}\-${id}\/${lastCheckpointId}`;
        };
        this.updateApiToken = (ev) => {
            const apiToken = ev.target.value;
            this.localConnection.updateParams({ apiToken });
            this.setState({ apiToken });
        };
        this.updateSelectedProject = (ev) => {
            const project = ev.target.value;
            this.localConnection.updateParams({ project });
            this.setState({ selectedProject: project });
        };
        this.backToConfigurationStep = () => {
            this.setState({ visibleStep: STEP.CONFIGURATION });
        };
        this.completeConfigurationStep = () => {
            this.setState({ visibleStep: STEP.INTEGRATION });
        };
        this.createNotebook = () => {
            this.setState({ error: null });
            this.props.onCreating();
            Promise.all([
                this.localConnection.getNotebook(),
                this.localConnection.getUsername(),
                this.localConnection.listProjects(),
                this.content.getMetadata(),
            ])
                .then(([notebook, username, projects, metadata]) => {
                if (notebook.id === metadata.notebookId) {
                    if (notebook.owner === username) {
                        const project = projects.find(project => project.id === notebook.projectId);
                        if (`${project.organizationName}/${project.name}` === this.localConnection.getParams().project) {
                            if (this.content.getNotebookPath() === notebook.path) {
                                return this.askUser('Nothing has changed.', notebook.id);
                            }
                            else {
                                return this.askUser('Notebook path has changed.', notebook.id);
                            }
                        }
                        else {
                            return this.askUser('Project has changed.', notebook.id);
                        }
                    }
                }
                return this.createNotebookFlow();
            })
                .catch(error => {
                this.props.onCreateFail();
                this.setState({ error });
            })
                .then(() => {
                this.localConnection
                    .getNotebook()
                    .then(notebook => {
                    this.setState({ notebook });
                    window.localStorage.setItem('neptuneLabs:projectIdentifier', this.state.selectedProject);
                });
            });
        };
        this.updateResolveStrategy = (conflictResolveStrategy) => {
            this.setState({ conflictResolveStrategy });
        };
        this.askUser = (textPrompt, notebookId) => {
            return apputils_1.showDialog({
                body: React.createElement(upload_1.UploadDialog, { header: textPrompt, onUpdateResolveStrategyChange: this.updateResolveStrategy }),
                buttons: [
                    apputils_1.Dialog.createButton({ label: 'Cancel', accept: false }),
                    apputils_1.Dialog.createButton({ label: 'Apply', accept: true })
                ]
            })
                .then(result => {
                if (result.button.accept) {
                    if (this.state.conflictResolveStrategy === upload_1.STRATEGY.create) {
                        return this.createNotebookFlow();
                    }
                    else {
                        return this.createCheckpointFlow(notebookId);
                    }
                }
                else {
                    return Promise.reject();
                }
            });
        };
        this.createCheckpointFlow = (notebookId) => {
            this.localConnection.updateParams({ notebookId });
            return this.content
                .getNotebookContent()
                .then(content => this.localConnection.createCheckpoint(this.content.getNotebookPath(), content))
                .then(() => {
                this.props.onCreateNotebook(this.localConnection.getParams());
                this.completeConfigurationStep();
                this.setState({ notebookId });
            });
        };
        this.createNotebookFlow = () => {
            return this.localConnection
                .createNotebook(this.content.getNotebookPath())
                .then(this.createCheckpointFlow);
        };
        this.integrateNotebook = () => {
            this
                .runCell()
                .then(() => {
                this.props.onClose();
            })
                .catch(err => {
                console.error('Integration failed', err);
            });
        };
        const { content, initParams, session, isConfigurationValid } = this.props;
        this.content = content;
        this.session = session;
        this.localConnection = new connection_1.NeptuneConnection(Object.assign({}, initParams));
        const { apiToken, project, notebookId } = this.localConnection.getParams();
        this.state = {
            error: null,
            visibleStep: isConfigurationValid ? STEP.INTEGRATION : STEP.CONFIGURATION,
            apiToken,
            selectedProject: project,
            projectsList: null,
            notebookId
        };
    }
    componentDidMount() {
        this.initializeDialog();
    }
    componentDidUpdate(prevProps, prevState) {
        if (prevState.apiToken !== this.state.apiToken) {
            this.initializeDialog();
        }
        if (prevProps.isOpen !== this.props.isOpen
            && this.props.isOpen) {
            this.setState({
                visibleStep: this.props.isConfigurationValid ? STEP.INTEGRATION : STEP.CONFIGURATION
            });
        }
    }
    renderCurrentStepHeader() {
        let configureStepState = 'current';
        let configureAction = null;
        let configureStepNo = React.createElement("span", null, "1");
        let integrateStepState = '';
        if (this.state.visibleStep === STEP.INTEGRATION) {
            configureStepState = 'completed';
            configureAction = this.backToConfigurationStep;
            configureStepNo = React.createElement("i", { className: "fa fa-check" });
            integrateStepState = 'current';
        }
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: `n-ConfigurationStep n-ConfigurationStep--${configureStepState}`, onClick: configureAction },
                React.createElement("span", { className: `n-ConfigurationStep__stepNo n-ConfigurationStep__stepNo--${configureStepState}` }, configureStepNo),
                "Checkpoints"),
            React.createElement("div", { className: "n-ConfigurationStepSeparator" }),
            React.createElement("div", { className: `n-ConfigurationStep n-ConfigurationStep--${integrateStepState}` },
                React.createElement("span", { className: `n-ConfigurationStep__stepNo n-ConfigurationStep__stepNo--${integrateStepState}` }, "2"),
                "Integration")));
    }
    renderCurrentStepContent() {
        if (this.state.visibleStep === STEP.CONFIGURATION) {
            return this.renderConfigureStep();
        }
        return this.renderIntegrateStep();
    }
    renderConfigureStep() {
        const { apiToken, error } = this.state;
        return (React.createElement(React.Fragment, null,
            React.createElement("div", { className: "n-ConfigurationStepLead" }, "API token and Project allows you to checkpoint and share your work in Neptune."),
            React.createElement("label", { className: "n-form-label", htmlFor: "neptune-api-token" }, "API token"),
            React.createElement("div", { className: "n-form-item" },
                React.createElement("input", { id: "neptune-api-token", className: "n-form-input n-form-input-with-status", type: "text", defaultValue: apiToken, title: apiToken, onChange: this.updateApiToken }),
                this.renderApiTokenStatus()),
            React.createElement("label", { className: "n-form-label", htmlFor: "neptune-project" }, "Select project"),
            React.createElement("div", { className: "n-form-item" },
                this.renderProjectsSelect(),
                this.renderProjectsStatus()),
            error && (React.createElement("div", { className: "n-form-error" }, error))));
    }
    renderInputStatus(condition) {
        const glyphClass = condition ? 'fa-check-circle' : 'fa-times-circle';
        let iconVariant = '';
        if (condition === true) {
            iconVariant = 'n-input-status-icon--valid';
        }
        else if (condition === false) {
            iconVariant = 'n-input-status-icon--invalid';
        }
        const iconClasses = 'n-input-status-icon fa fa-lg ' + glyphClass + ' ' + iconVariant;
        return (React.createElement("i", { id: "neptune-api-token-icon", className: iconClasses }));
    }
    ;
    renderIntegrateStep() {
        const { apiToken, selectedProject, notebookId, notebook } = this.state;
        const code = kernel_1.getInitializationCode({
            apiToken,
            project: selectedProject,
            notebookId
        });
        return (React.createElement(React.Fragment, null,
            (notebook) && (React.createElement("div", { className: "n-ConfigurationStepLead n-ConfigurationStepLead-success" },
                "Initial checkpoint successful! Check ",
                React.createElement("a", { href: this.getNotebookURI(), target: "_blank", rel: "noopener noreferrer" }, "this link"),
                " to see your notebook ",
                React.createElement("em", null, notebook.name),
                ".")),
            React.createElement("div", { className: "n-form-label n-form-label--small" }, "Integrate to create Neptune experiments and see them all linked to this notebook. Click \"Integrate\" to run the code below, then just \"import neptune\" and work as usual."),
            React.createElement("div", { className: "n-form-item" },
                React.createElement(react_codemirror2_1.UnControlled, { className: "notebook-integration-code", options: {
                        mode: 'python',
                        readOnly: true
                    }, value: code }))));
    }
    renderCurrentStepActions() {
        const { onClose } = this.props;
        if (this.state.visibleStep === STEP.CONFIGURATION) {
            const requireParams = !this.state.isApiTokenValid || !this.state.selectedProject || this.props.isLoading;
            const disabledClass = requireParams ? 'is-disabled' : '';
            return (React.createElement(React.Fragment, null,
                React.createElement("button", { type: "button", className: "n-ConfigureModalButton n-ConfigureModalButton--secondary", onClick: onClose }, "Cancel"),
                React.createElement("button", { type: "button", className: "n-ConfigureModalButton n-ConfigureModalButton--primary " + disabledClass, onClick: this.createNotebook, disabled: requireParams }, "Create notebook")));
        }
        return (React.createElement(React.Fragment, null,
            React.createElement("button", { type: "button", className: "n-ConfigureModalButton n-ConfigureModalButton--secondary", onClick: onClose }, "Cancel"),
            React.createElement("button", { type: "button", className: "n-ConfigureModalButton n-ConfigureModalButton--primary", onClick: this.integrateNotebook }, "Integrate")));
    }
    runCell() {
        return this.session.runInitializationCode(this.localConnection.getParams()).done;
    }
    ;
    render() {
        const { isOpen } = this.props;
        return (React.createElement(ReactModal, { isOpen: isOpen, ariaHideApp: false, className: "n-ConfigureModal", overlayClassName: "n-ConfigureModalOverlay" },
            React.createElement("div", { className: "n-ConfigureModalHeader" }, this.renderCurrentStepHeader()),
            React.createElement("div", { className: "n-ConfigureModalBody" }, this.renderCurrentStepContent()),
            React.createElement("div", { className: "n-ConfigureModalFooter" }, this.renderCurrentStepActions())));
    }
}
exports.ConfigureModal = ConfigureModal;
//# sourceMappingURL=configure-modal.js.map