"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@phosphor/disposable");
const content_1 = require("./content");
const kernel_1 = require("./kernel");
const connection_1 = require("./connection");
const configure_button_1 = require("./configure-button");
const upload_1 = require("./upload");
class NeptuneNotebookExtension {
    constructor(app) {
        this.app = app;
    }
    createNew(panel, context) {
        let idx = 0;
        let namesIt = panel.toolbar.names();
        let name;
        while ((name = namesIt.next()) && name !== undefined && name != 'spacer') {
            idx++;
        }
        let content = new content_1.NeptuneContent(context, this.app.serviceManager.contents);
        let session = new kernel_1.NeptuneSession(context.session);
        let buttonsPromise = content
            .getMetadata()
            .then(metadata => {
            return connection_1.createConnection({
                apiToken: connection_1.getGlobalApiToken(),
                project: window.localStorage.getItem('neptuneLabs:projectIdentifier'),
                notebookId: metadata.notebookId
            })
                .catch(() => Promise.resolve(connection_1.createEmptyConnection()))
                .then(connection => {
                const neptuneConfigureButton = new configure_button_1.NeptuneConfigureButton(content, session, connection);
                const neptuneUploadNotebookButton = new upload_1.NeptuneUploadButton(content, connection);
                panel.toolbar.insertItem(idx++, 'neptune:configure', neptuneConfigureButton);
                panel.toolbar.insertItem(idx, 'neptune:uploadNotebook', neptuneUploadNotebookButton);
                return [
                    neptuneConfigureButton,
                    neptuneUploadNotebookButton
                ];
            });
        });
        return new disposable_1.DisposableDelegate(() => {
            return buttonsPromise.then(buttons => buttons.forEach(button => button.dispose()));
        });
    }
}
exports.NeptuneNotebookExtension = NeptuneNotebookExtension;
const plugin = {
    id: 'neptune-notebook',
    autoStart: true,
    activate: (app) => {
        app.docRegistry.addWidgetExtension('Notebook', new NeptuneNotebookExtension(app));
    }
};
exports.default = plugin;
//# sourceMappingURL=index.js.map